//  SuperTuxKart - a fun racing game with go-kart
//  Copyright (C) 2006 SuperTuxKart-Team
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include "modes/standard_race.hpp"

#include "challenges/unlock_manager.hpp"
#include "config/user_config.hpp"

//-----------------------------------------------------------------------------
StandardRace::StandardRace() : LinearWorld()
{
    TimedRace::setClockMode(CHRONO);
}   // StandardRace

//-----------------------------------------------------------------------------
StandardRace::~StandardRace()
{
}   // ~StandardRace
    
#if 0
#pragma mark -
#pragma mark clock events
#endif

//-----------------------------------------------------------------------------
void StandardRace::onGo()
{
    // Reset the brakes now that the prestart 
    // phase is over (braking prevents the karts 
    // from sliding downhill)
    for(unsigned int i=0; i<m_kart.size(); i++) 
    {
        m_kart[i]->resetBrakes();
    }
}   // onGo

//-----------------------------------------------------------------------------
void StandardRace::terminateRace()
{
    LinearWorld::terminateRace();
}   // terminateRace

#if 0
#pragma mark -
#pragma mark overridden from World
#endif

//-----------------------------------------------------------------------------
void StandardRace::restartRace()
{
    LinearWorld::restartRace();
}   // restartRace

//-----------------------------------------------------------------------------
void StandardRace::update(float delta)
{    
    LinearWorld::update(delta);
    if(!TimedRace::isRacePhase()) return;
    
    // All karts are finished
    if(race_manager->getFinishedKarts() >= race_manager->getNumKarts() )
    {
        enterRaceOverState();
        unlock_manager->raceFinished();
    }   // if all karts are finished
    
    // All player karts are finished, but computer still racing
    // ===========================================================
    else if(isRaceOver())
    {
        // Set delay mode to have time for camera animation, and
        // to give the AI some time to get non-estimated timings
        enterRaceOverState(true /* delay */);
    }
}   // update

//-----------------------------------------------------------------------------
/** Returns tru if the race is finished, i.e. all player karts are finished.
 */
bool StandardRace::isRaceOver()
{
    return race_manager->allPlayerFinished();
}   // isRaceOver

//-----------------------------------------------------------------------------
void StandardRace::getDefaultCollectibles(int& collectible_type, int& amount)
{
    // in time trial mode, give zippers
    if(race_manager->getMinorMode() == RaceManager::MINOR_MODE_TIME_TRIAL)
    {
        collectible_type = POWERUP_ZIPPER;
        amount = race_manager->getNumLaps();
    }
    else World::getDefaultCollectibles(collectible_type, amount);
}   // getDefaultCollectibles

//-----------------------------------------------------------------------------
/** Returns if this mode supports bonus boxes or not.
 */
bool StandardRace::haveBonusBoxes()
{
    // in time trial mode, don't use bonus boxes
    return race_manager->getMinorMode() != RaceManager::MINOR_MODE_TIME_TRIAL;
}   // haveBonusBoxes

//-----------------------------------------------------------------------------
/** Returns an identifier for this race. 
 */
std::string StandardRace::getIdent() const
{
    if(race_manager->getMinorMode() == RaceManager::MINOR_MODE_TIME_TRIAL)
        return IDENT_TTRIAL;
    else
        return IDENT_STD;
    
}   // getIdent
